@@ -1,6 +1,6 @@
 if(getRversion() >= "2.15.1") utils::globalVariables("UNlocations")
 
-pop.predict <- function(end.year=2100, start.year=1950, present.year=2010, wpp.year=2010,
+pop.predict <- function(end.year=2100, start.year=1950, present.year=2010, wpp.year=2012,
 						countries=NULL, output.dir = file.path(getwd(), "bayesPop.output"),
 						inputs=list(
 							popM=NULL,
@@ -13,7 +13,7 @@ pop.predict <- function(end.year=2100, start.year=1950, present.year=2010, wpp.y
 							migM=NULL,
 							migF=NULL,	
 							e0F.file=NULL, e0M.file=NULL, 
-							tfr.file=NULL,
+							tfr.file=NULL, 
 							e0F.sim.dir=NULL, e0M.sim.dir=NULL, 
 							tfr.sim.dir=NULL	
 						), nr.traj = 1000, keep.vital.events=FALSE,
@@ -321,7 +321,7 @@ load.inputs <- function(inputs, start.year, present.year, end.year, wpp.year) {
 	num.columns <- grep('^[0-9]{4}$', colnames(POPm0), value=TRUE) # values of year-columns
 	if(!is.element(as.character(present.year), colnames(POPm0))) {
 		stop('Wrong present.year. ', present.year, ' not available in the popM file.\nAvailable years: ',
-				num.columns)
+				paste(num.columns, collapse=', '))
 	}
 	num.columns <- num.columns[which(as.integer(num.columns)<= present.year)]
 	popm.matrix <- POPm0[,num.columns]
@@ -335,7 +335,7 @@ load.inputs <- function(inputs, start.year, present.year, end.year, wpp.year) {
 	num.columns <- grep('^[0-9]{4}$', colnames(POPf0), value=TRUE) # values of year-columns
 	if(!is.element(as.character(present.year), colnames(POPf0))) {
 		stop('Wrong present.year. ', present.year, ' not available in the popF file.\nAvailable years: ',
-				num.columns)
+				paste(num.columns, collapse=', '))
 	}
 	num.columns <- num.columns[which(as.integer(num.columns)<= present.year)]
 	popf.matrix <- POPf0[, num.columns]
@@ -418,17 +418,20 @@ load.inputs <- function(inputs, start.year, present.year, end.year, wpp.year) {
 	if(!is.null(inputs$e0F.sim.dir))  # female
 		e0Fpred <- get.e0.prediction(inputs$e0F.sim.dir, mcmc.dir=NA)
 	else {
-		file.name <- if(!is.null(inputs$e0F.file)) inputs$e0F.file
-					else file.path(find.package("bayesPop"), "ex-data", 
-							paste('e0Fwpp', wpp.year, '.csv', sep=''))
-		if(!file.exists(file.name))
-			stop('File ', file.name, 
-				' does not exist.\nSet e0F.sim.dir, e0F.file or change WPP year.')
-		e0Fpred <- read.csv(file=file.name, comment.char='#', check.names=FALSE)
-		e0Fpred <- e0Fpred[,c('LocID', 'Year', 'e0')]
+		if(!is.null(inputs$e0F.file)) {
+			file.name <-  inputs$e0F.file
+			if(!file.exists(file.name))
+				stop('File ', file.name, 
+					' does not exist.\nSet e0F.sim.dir, e0F.file or change WPP year.')
+			 # comma separated trajectories file
+			e0Fpred <- read.csv(file=file.name, comment.char='#', check.names=FALSE)
+			e0Fpred <- e0Fpred[,c('LocID', 'Year', 'e0')]
+		} else {
+						#file.path(find.package("bayesPop"), "ex-data", 
+						 #	paste('e0Fwpp', wpp.year, '.csv', sep=''))
+			e0Fpred <- .load.wpp.traj('e0F', wpp.year)
+		}
 		colnames(e0Fpred) <- c('country_code', 'year', 'value')
-		#observed$e0Fpred <- e0Fpred[e0Fpred[,'year']+2<present.year,]
-		#e0Fpred <- e0Fpred[e0Fpred[,'year']+2>=present.year,]
 	} 
 
 	if(!is.null(inputs$e0M.sim.dir)) { # male
@@ -438,36 +441,37 @@ load.inputs <- function(inputs, start.year, present.year, end.year, wpp.year) {
 			e0Mpred <- get.e0.jmale.prediction(e0Fpred)
 		} else e0Mpred <- get.e0.prediction(inputs$e0M.sim.dir, mcmc.dir=NA)
 	} else {
-		file.name <- if(!is.null(inputs$e0M.file)) inputs$e0M.file 
-					else file.path(find.package("bayesPop"), "ex-data", 
-							paste('e0Mwpp', wpp.year, '.csv', sep=''))
-		if(!file.exists(file.name))
-			stop('File ', file.name, 
-				' does not exist.\nSet e0M.sim.dir, e0M.file or change WPP year.')
-		e0Mpred <- read.csv(file=file.name, comment.char='#', check.names=FALSE)
-		e0Mpred <- e0Mpred[,c('LocID', 'Year', 'e0')]
+		if(!is.null(inputs$e0M.file)) {
+			file.name <-  inputs$e0M.file 
+					#else file.path(find.package("bayesPop"), "ex-data", 
+					#		paste('e0Mwpp', wpp.year, '.csv', sep=''))
+			if(!file.exists(file.name)) 
+				stop('File ', file.name, 
+					' does not exist.\nSet e0M.sim.dir, e0M.file or change WPP year.')
+			e0Mpred <- read.csv(file=file.name, comment.char='#', check.names=FALSE)
+			e0Mpred <- e0Mpred[,c('LocID', 'Year', 'e0')]
+		} else
+			e0Mpred <- .load.wpp.traj('e0M', wpp.year)
 		colnames(e0Mpred) <- c('country_code', 'year', 'value')
-		#observed$e0Mpred <- e0Mpred[e0Mpred[,'year']+2<present.year,]
-		#e0Mpred <- e0Mpred[e0Mpred[,'year']+2>=present.year,]
-
 	} 
 		
 	# Get TFR
 	if(!is.null(inputs$tfr.sim.dir)) 
 		TFRpred <- get.tfr.prediction(inputs$tfr.sim.dir, mcmc.dir=NA)
 	else {
-		file.name <- if(!is.null(inputs$tfr.file)) inputs$tfr.file
-					else file.path(find.package("bayesPop"), "ex-data", 
-							paste('TFRwpp', wpp.year, '.csv', sep=''))
-		if(!file.exists(file.name))
-			stop('File ', file.name, 
-				' does not exist.\nSet tfr.sim.dir, tfr.file or change WPP year.')
-		TFRpred <- read.csv(file=file.name, comment.char='#', check.names=FALSE)
-		TFRpred <- TFRpred[,c('LocID', 'Year', 'TF')]
+		if(!is.null(inputs$tfr.file)) {
+			file.name <- inputs$tfr.file
+					#else file.path(find.package("bayesPop"), "ex-data", 
+					#		paste('TFRwpp', wpp.year, '.csv', sep=''))
+			if(!file.exists(file.name))
+				stop('File ', file.name, 
+					' does not exist.\nSet tfr.sim.dir, tfr.file or change WPP year.')
+			TFRpred <- read.csv(file=file.name, comment.char='#', check.names=FALSE)
+			TFRpred <- TFRpred[,c('LocID', 'Year', 'TF')]
+		} else 
+			TFRpred <- .load.wpp.traj('tfr', wpp.year)
 		colnames(TFRpred) <- c('country_code', 'year', 'value')
-		#observed$TFRpred <- TFRpred[TFRpred[,'year']+2<present.year,]
-		#TFRpred <- TFRpred[TFRpred[,'year']+2>=present.year,]
-	} 
+	}
 	return(list(POPm0=POPm0, POPf0=POPf0, MXm=MXm, MXf=MXf, SRB=SRB,
 				PASFR=PASFR, MIGtype=MIGtype, MIGm=MIGm, MIGf=MIGf,
 				e0Mpred=e0Mpred, e0Fpred=e0Fpred, TFRpred=TFRpred, 
@@ -477,6 +481,51 @@ load.inputs <- function(inputs, start.year, present.year, end.year, wpp.year) {
 				observed=observed))
 }
 
+.load.wpp.traj <- function(type, wpp.year) {
+	dataset.obs <- dataset.low <- dataset.high <- NA
+	if(type %in% c('e0F', 'e0M')){
+		if(wpp.year < 2010) stop('e0 projections not available for wpp 2008.')
+		#if(wpp.year == 2010) 
+		dataset <- paste(type, 'proj', sep='')
+		#else { # wpp 2012
+		#	dataset <- paste(type, 'projMed', sep='')
+		#}
+		dataset.obs <- type
+	} else { # tfr
+		if(wpp.year < 2010) dataset <- type
+		else {
+			dataset <- paste(type, 'projMed', sep='')
+			dataset.obs <- type
+			dataset.low <- paste(type, 'projLow', sep='')
+			dataset.high <- paste(type, 'projHigh', sep='')
+		}
+	}
+	if(!is.na(dataset.obs)) 
+		pred.obs <- bayesTFR:::load.bdem.dataset(dataset.obs, wpp.year)
+		
+	pred.all <- NULL
+	for(dataset.name in c(dataset, dataset.low, dataset.high)) {
+		if(is.na(dataset.name)) next
+		pred <- bayesTFR:::load.bdem.dataset(dataset.name, wpp.year)
+		remove.cols <- which(colnames(pred) %in% c('name', 'country', 'last.observed'))
+		pred <- pred[,-remove.cols]
+		if(!is.na(dataset.obs)) {
+			remove.cols <- which(colnames(pred.obs) %in% c('name', 'country', 'last.observed', 
+								colnames(pred)[-which(colnames(pred)=='country_code')]))
+			pred <- merge(pred.obs[,-remove.cols], pred, by='country_code')
+		}
+		ncols <- ncol(pred)
+		nonnum.idx <- which(colnames(pred)=='country_code')
+		cnames <- colnames(pred)[-nonnum.idx]
+		colnames(pred)[-nonnum.idx] <- paste(type, cnames, sep='')
+		pred.long <- reshape(pred, direction='long', varying=(1:ncols)[-nonnum.idx], v.names=type, times=cnames)
+		pred.long <- cbind(pred.long, year=as.integer(substr(pred.long$time,1,4))+3)
+		pred.long <- pred.long[,c('country_code', 'year', type)]
+		pred.all <- rbind(pred.all, pred.long)
+	}
+	return(pred.all)
+}
+
 .get.par.from.inputs <- function(par, inputs, country) {
 	if(is.null(inputs[[par]])) return (NULL)
 	idx <- inputs[[par]][,'country_code'] == country
@@ -501,6 +550,7 @@ get.country.inputs <- function(country, inputs, nr.traj, country.name) {
 			cidx <- inputs[[par]][,'country_code'] == country 
 			idx <- cidx & is.element(inputs[[par]][,'year'], inputs$proj.years)
 			if(sum(idx) == 0) {
+				stop('')
 				warning('No ', what.traj[[par]], ' trajectories for ', country.name, 
 					'. No population projection generated.')
 				return(NULL)
@@ -573,12 +623,17 @@ get.country.inputs <- function(country, inputs, nr.traj, country.name) {
 	indices$e0Fpred <- if (ncol(inpc$e0Mpred) != ncol(inpc$e0Fpred)) get.traj.index(nr.traj, inpc$e0Fpred)
 					else indices$e0Mpred
 	indices$TFRpred <- get.traj.index(nr.traj, inpc$TFRpred)
-	nr.traj <- min(length(indices$e0Mpred), length(indices$e0Fpred), length(indices$TFRpred))
+	nr.traj <- max(length(indices$e0Mpred), length(indices$e0Fpred), length(indices$TFRpred))
 	for (par in c('TFRpred', 'e0Mpred', 'e0Fpred')) {
-		if(length(indices[[par]]) != nr.traj)
+		if(length(indices[[par]]) > nr.traj)
 			indices[[par]] <- get.traj.index(nr.traj, inpc[[par]])
+		else {
+			if(length(indices[[par]]) < nr.traj) {
+				indices[[par]] <- sample(indices[[par]], nr.traj, replace=TRUE)
+			}
+		}
 	}
-			
+
 	if(length(indices$TFRpred) != length(indices$e0Mpred) || length(indices$e0Mpred) != length(indices$e0Fpred)) {
 		warning('Mismatch in number of trajectories of TFR and/or life expactancy for ', 
 						country.name, '. No population projection generated.')
